'use strict';

var jsxRuntime = require('react/jsx-runtime');
var react = require('react');
var RFB = require('@novnc/novnc/core/rfb');

var VncViewer = function (_a, ref) {
    var url = _a.url, style = _a.style, className = _a.className, viewOnly = _a.viewOnly, rfbOptions = _a.rfbOptions, focusOnClick = _a.focusOnClick, clipViewport = _a.clipViewport, dragViewport = _a.dragViewport, scaleViewport = _a.scaleViewport, resizeSession = _a.resizeSession, showDotCursor = _a.showDotCursor, background = _a.background, qualityLevel = _a.qualityLevel, compressionLevel = _a.compressionLevel, _b = _a.autoConnect, autoConnect = _b === void 0 ? true : _b, _c = _a.retryDuration, retryDuration = _c === void 0 ? 3000 : _c, _d = _a.debug, debug = _d === void 0 ? false : _d, loader = _a.loader, onConnect = _a.onConnect, onDisconnect = _a.onDisconnect, onCredentialsRequired = _a.onCredentialsRequired, onSecurityFailure = _a.onSecurityFailure, onClipboard = _a.onClipboard, onBell = _a.onBell, onDesktopName = _a.onDesktopName, onCapabilities = _a.onCapabilities;
    var rfb = react.useRef(null);
    var connected = react.useRef(autoConnect);
    var timeouts = react.useRef([]);
    var eventListeners = react.useRef({});
    var screen = react.useRef(null);
    var _e = react.useState(true), loading = _e[0], setLoading = _e[1];
    var logger = {
        log: function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (debug)
                console.log.apply(console, args);
        },
        info: function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (debug)
                console.info.apply(console, args);
        },
        error: function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (debug)
                console.error.apply(console, args);
        },
    };
    var getRfb = function () { return rfb.current; };
    var setRfb = function (_rfb) { return (rfb.current = _rfb); };
    var getConnected = function () { return connected.current; };
    var setConnected = function (state) { return (connected.current = state); };
    var _onConnect = function () {
        var rfb = getRfb();
        if (onConnect) {
            onConnect(rfb !== null && rfb !== void 0 ? rfb : undefined);
            setLoading(false);
        }
        logger.info('Connected to remote VNC.');
        setLoading(false);
    };
    var _onDisconnect = function () {
        var rfb = getRfb();
        if (onDisconnect) {
            onDisconnect(rfb !== null && rfb !== void 0 ? rfb : undefined);
            setLoading(true);
        }
        var connected = getConnected();
        if (connected) {
            logger.info("Unexpectedly disconnected from remote VNC, retrying in ".concat(retryDuration / 1000, " seconds."));
            timeouts.current.push(setTimeout(connect, retryDuration));
        }
        else {
            logger.info('Disconnected from remote VNC.');
        }
        setLoading(true);
    };
    var _onCredentialsRequired = function () {
        var _a, _b;
        var rfb = getRfb();
        if (onCredentialsRequired)
            onCredentialsRequired(rfb !== null && rfb !== void 0 ? rfb : undefined);
        var password = (_b = (_a = rfbOptions === null || rfbOptions === void 0 ? void 0 : rfbOptions.credentials) === null || _a === void 0 ? void 0 : _a.password) !== null && _b !== void 0 ? _b : prompt('Password Required:');
        if (password)
            rfb === null || rfb === void 0 ? void 0 : rfb.sendCredentials({ password: password });
    };
    var _onDesktopName = function (e) {
        if (onDesktopName)
            onDesktopName(e);
        logger.info("Desktop name is ".concat(e.detail.name));
    };
    var disconnect = function () {
        var rfb = getRfb();
        try {
            if (rfb) {
                timeouts.current.forEach(function (id) { return clearTimeout(id); });
                Object.keys(eventListeners.current).forEach(function (event) {
                    if (eventListeners.current[event]) {
                        rfb === null || rfb === void 0 ? void 0 : rfb.removeEventListener(event, eventListeners.current[event]);
                        eventListeners.current[event] = undefined;
                    }
                });
                rfb.disconnect();
                rfb = null;
                setRfb(null);
                setConnected(false);
                // NOTE: This needs to be called since the event listener is removed.
                // Even if the event listener is removed after rfb.disconnect(), the disconnect
                // event is not fired.
                _onDisconnect();
            }
        }
        catch (error) {
            logger.error(error);
            setRfb(null);
            setConnected(false);
        }
    };
    //eslint-disable-next-line sonarjs/cognitive-complexity
    var connect = function () {
        try {
            if (connected && !!rfb)
                disconnect();
            if (screen.current) {
                screen.current.innerHTML = '';
                var _rfb_1 = new RFB(screen.current, url, rfbOptions);
                _rfb_1.viewOnly = viewOnly !== null && viewOnly !== void 0 ? viewOnly : false;
                _rfb_1.focusOnClick = focusOnClick !== null && focusOnClick !== void 0 ? focusOnClick : false;
                _rfb_1.clipViewport = clipViewport !== null && clipViewport !== void 0 ? clipViewport : false;
                _rfb_1.dragViewport = dragViewport !== null && dragViewport !== void 0 ? dragViewport : false;
                _rfb_1.resizeSession = resizeSession !== null && resizeSession !== void 0 ? resizeSession : false;
                _rfb_1.scaleViewport = scaleViewport !== null && scaleViewport !== void 0 ? scaleViewport : false;
                _rfb_1.showDotCursor = showDotCursor !== null && showDotCursor !== void 0 ? showDotCursor : false;
                _rfb_1.background = background !== null && background !== void 0 ? background : '';
                _rfb_1.qualityLevel = qualityLevel !== null && qualityLevel !== void 0 ? qualityLevel : 6;
                _rfb_1.compressionLevel = compressionLevel !== null && compressionLevel !== void 0 ? compressionLevel : 2;
                setRfb(_rfb_1);
                eventListeners.current.connect = _onConnect;
                eventListeners.current.disconnect = _onDisconnect;
                eventListeners.current.credentialsrequired = _onCredentialsRequired;
                eventListeners.current.securityfailure = onSecurityFailure;
                eventListeners.current.clipboard = onClipboard;
                eventListeners.current.bell = onBell;
                eventListeners.current.desktopname = _onDesktopName;
                eventListeners.current.capabilities = onCapabilities;
                Object.keys(eventListeners.current).forEach(function (event) {
                    if (eventListeners.current[event]) {
                        _rfb_1.addEventListener(event, eventListeners.current[event]);
                    }
                });
                setConnected(true);
            }
        }
        catch (error) {
            logger.error(error);
        }
    };
    var sendCredentials = function (credentials) {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.sendCredentials(credentials);
    };
    var sendKey = function (keysym, code, down) {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.sendKey(keysym, code, down);
    };
    var sendCtrlAltDel = function () {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.sendCtrlAltDel();
    };
    var focus = function () {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.focus();
    };
    var blur = function () {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.blur();
    };
    var machineShutdown = function () {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.machineShutdown();
    };
    var machineReboot = function () {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.machineReboot();
    };
    var machineReset = function () {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.machineReset();
    };
    var clipboardPaste = function (text) {
        var rfb = getRfb();
        rfb === null || rfb === void 0 ? void 0 : rfb.clipboardPasteFrom(text);
    };
    react.useImperativeHandle(ref, function () { return ({
        connect: connect,
        disconnect: disconnect,
        connected: connected.current,
        sendCredentials: sendCredentials,
        sendKey: sendKey,
        sendCtrlAltDel: sendCtrlAltDel,
        focus: focus,
        blur: blur,
        machineShutdown: machineShutdown,
        machineReboot: machineReboot,
        machineReset: machineReset,
        clipboardPaste: clipboardPaste,
        rfb: rfb.current,
        eventListeners: eventListeners.current,
    }); });
    react.useEffect(function () {
        if (autoConnect)
            connect();
        return disconnect;
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    var handleClick = function () {
        var rfb = getRfb();
        if (rfb)
            rfb.focus();
    };
    var handleMouseEnter = function () {
        if (document.activeElement && document.activeElement instanceof HTMLElement)
            document.activeElement.blur();
        handleClick();
    };
    var handleMouseLeave = function () {
        var rfb = getRfb();
        if (rfb)
            rfb.blur();
    };
    return (jsxRuntime.jsxs(jsxRuntime.Fragment, { children: [(loading || !url) && (loader !== null && loader !== void 0 ? loader : jsxRuntime.jsx("p", { children: "Loading..." })), url && (jsxRuntime.jsx("div", { style: style, className: className, ref: screen, onMouseEnter: handleMouseEnter, onMouseLeave: handleMouseLeave }))] }));
};
var VncViewer$1 = react.forwardRef(VncViewer);

exports.Events = void 0;
(function (Events) {
    Events[Events["connect"] = 0] = "connect";
    Events[Events["disconnect"] = 1] = "disconnect";
    Events[Events["credentialsrequired"] = 2] = "credentialsrequired";
    Events[Events["securityfailure"] = 3] = "securityfailure";
    Events[Events["clipboard"] = 4] = "clipboard";
    Events[Events["bell"] = 5] = "bell";
    Events[Events["desktopname"] = 6] = "desktopname";
    Events[Events["capabilities"] = 7] = "capabilities";
})(exports.Events || (exports.Events = {}));

exports.VncViewer = VncViewer$1;
